import sys
from imaplib import IMAP4_SSL
from email import policy
from email import message_from_bytes
from email.header import decode_header

from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.exception.exception_tags import mail_thunder_content_login_failed


class IMAPWrapper(IMAP4_SSL):

    def __init__(self, host: str = 'imap.gmail.com'):
        super().__init__(host)
        user_info = read_output_content()
        try:
            if user_info is not None and type(user_info) == dict:
                if "user" in user_info.keys() and "password" in user_info.keys():
                    if user_info.get("user") is not None and user_info.get("password") is not None:
                        self.login(user_info.get("user"), user_info.get("password"))
        except Exception as error:
            print(repr(error) + " " + mail_thunder_content_login_failed, file=sys.stderr)

    def search_mailbox(self, search_str: [str, list] = "ALL", charset: str = None):
        response, mail_number_string = self.search(charset, search_str)
        mail_detail_list = list()
        for num_of_mail in mail_number_string[0].split():
            response, mail_data = self.fetch(num_of_mail, "(RFC822)")
            # [0][1] is message data [0][0] is message decode like RFC822 {565}
            message = message_from_bytes(mail_data[0][1], policy=policy.default)
            mail_detail_list.append([response, mail_data[0][0], message])
        return mail_detail_list

    def mail_content_list(self, search_str: [str, list] = "ALL", charset: str = None):
        mail_list = self.search_mailbox(search_str, charset)
        mail_content_dict = dict()
        mail_content_list = list()
        for mail_data in mail_list:
            mail = mail_data[2]
            mail_content_dict.update({"SUBJECT": mail.get("Subject")})
            mail_content_dict.update({"FROM": mail.get("FROM")})
            mail_content_dict.update({"TO": mail.get("TO")})
            body = ""
            if mail.is_multipart():
                for part in mail.get_payload():
                    body = part.get_payload(decode=True)
            else:
                body = mail.get_payload(decode=True)

            body = str(decode_header(str(body))[0][0])
            mail_content_dict.update({"BODY": body})
            mail_content_list.append(mail_content_dict)
            mail_content_dict = dict()
        return mail_content_list




# argparse
import argparse
import json

from je_mail_thunder.smtp.smtp_wrapper import SMTPWrapper
from je_mail_thunder.utils.exception.exception_tags import mail_thunder_login_error, \
    mail_thunder_argparse_get_wrong_function
from je_mail_thunder.utils.exception.exceptions import MailThunderArgparseException

if __name__ == "__main__":
    smtp_service = SMTPWrapper()

    argparse_service_function_dict = {
        "send_mail": smtp_service.create_message_and_send,
        "send_mail_with_attach": smtp_service.create_message_with_attach_and_send,
        "login_with_env_or_content": smtp_service.try_to_login_with_env_or_content
    }

    all_param_required_function_list = ["send_mail", "send_mail_with_attach"]

    only_function_param_function_list = [
        "login_with_env_or_content"
    ]

    parser = argparse.ArgumentParser()
    parser.add_argument("--user", help="set email user")
    parser.add_argument("--password", help="set email password")
    parser.add_argument("--function", help="service function", required=True)
    parser.add_argument("--setting", help="service function setting as json")
    args = parser.parse_args()
    args = vars(args)
    if args.get("function") in all_param_required_function_list:
        setting = args.get("setting")
        setting = setting.replace("\"", "").replace("'", '"')
        for param in args.values():
            if param is None:
                raise MailThunderArgparseException(mail_thunder_login_error)
        smtp_service.login(args.get("user"), args.get("password"))
        argparse_service_function_dict.get(args.get("function"))(**json.loads(setting))
    elif args.get("function") in only_function_param_function_list:
        argparse_service_function_dict.get(args.get("function"))()
    else:
        raise MailThunderArgparseException(mail_thunder_argparse_get_wrong_function)



"""
json exception
"""
mail_thunder_cant_reformat_json_error = "Can't reformat json is type right?"
mail_thunder_wrong_json_data_error = "Can't parser json"
"""
content data error
"""
mail_thunder_content_set_compiler_error = "When set compiler using content make an error"
mail_thunder_content_file_error = "MailThunder content file error"
mail_thunder_content_login_failed = "can't login with mail thunder content"
"""
argparse
"""
mail_thunder_service_file_error = "service param -s got the wrong data"
mail_thunder_login_error = "need set --user and --password to login"
mail_thunder_argparse_get_wrong_function = "get unknown function"
"""
SMTP 
"""

"""
IMAP
"""



import smtplib
import sys
from email.message import EmailMessage
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from mimetypes import guess_type
from os import path
from smtplib import SMTP_SSL

from je_mail_thunder.utils.exception.exception_tags import mail_thunder_content_login_failed
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.save_mail_user_content.save_on_env import get_mail_thunder_os_environ


class SMTPWrapper(SMTP_SSL):

    def __init__(self, host: str = "smtp.gmail.com", port: int = 465):
        super().__init__(host, port)
        self.try_to_login_with_env_or_content()
        self.login_state = False

    @staticmethod
    def create_message(message_content: str, message_setting_dict: dict, **kwargs):
        message = EmailMessage(**kwargs)
        message.set_content(message_content)
        for key, value in message_setting_dict.items():
            message[key] = value
        return message

    @staticmethod
    def create_message_with_attach(message_content: str, message_setting_dict: dict,
                                   attach_file: str, use_html: bool = False):
        message = MIMEMultipart()
        for key, value in message_setting_dict.items():
            message[key] = value
        if use_html:
            mime_part = MIMEText(message_content, "html")
        else:
            mime_part = MIMEText(message_content)
        message.attach(mime_part)
        content_type, encoding = guess_type(attach_file)
        if content_type is None or encoding is not None:
            content_type = "application/octet-stream"
        main_type, sub_type = content_type.split("/", 1)
        if main_type == "text":
            file_read = open(attach_file, "r+")
            mime_part = MIMEText(file_read.read(), _subtype=sub_type)
            file_read.close()
        elif main_type == "image":
            file_read = open(attach_file, "rb")
            mime_part = MIMEImage(file_read.read(), _subtype=sub_type)
            file_read.close()
        elif main_type == "audio":
            file_read = open(attach_file, "rb")
            mime_part = MIMEAudio(file_read.read(), _subtype=sub_type)
            file_read.close()
        else:
            file_read = open(attach_file, "rb")
            mime_part = MIMEBase(main_type, sub_type)
            mime_part.set_payload(file_read.read())
            file_read.close()
        filename = path.basename(attach_file)
        mime_part.add_header("Content-Disposition", "attachment", filename=filename)
        mime_part.add_header("Content-ID", "{filename}".format(filename=filename))
        message.attach(mime_part)
        return message

    def try_to_login_with_env_or_content(self):
        user_info = read_output_content()
        self.login_state = False
        try:
            if user_info is not None and type(user_info) == dict:
                if user_info.get("user", None) is not None and user_info.get("password", None) is not None:
                    self.login(user_info.get("user"), user_info.get("password"))
                    self.login_state = True
            else:
                user_info = get_mail_thunder_os_environ()
                if user_info is not None and type(user_info) == dict:
                    if user_info.get("mail_thunder_user", None) is not None and user_info.get(
                            "mail_thunder_user_password", None) is not None:
                        self.login(user_info.get("mail_thunder_user"), user_info.get("mail_thunder_user_password"))
                        self.login_state = True
            return self.login_state
        except smtplib.SMTPAuthenticationError as error:
            print(repr(error) + " " + mail_thunder_content_login_failed, file=sys.stderr)
            return self.login_state

    def quit(self):
        self.login_state = False

    def create_message_with_attach_and_send(self, message_content: str, message_setting_dict: dict,
                                            attach_file: str, use_html: bool = False):

        self.send_message(self.create_message_with_attach(message_content, message_setting_dict, attach_file, use_html))

    def create_message_and_send(self, message_content: str, message_setting_dict: dict, **kwargs):
        self.send_message(self.create_message(message_content, message_setting_dict, **kwargs))



class MailThunderException(Exception):
    pass


class MailThunderJsonException(Exception):
    pass


class MailThunderContentException(Exception):
    pass


class MailThunderArgparseException(Exception):
    pass



# SMTP Wrapper
from je_mail_thunder.imap.imap_wrapper import IMAPWrapper
# IMAP Wrapper
from je_mail_thunder.smtp.smtp_wrapper import SMTPWrapper
# content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_data import is_need_to_save_content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import write_output_content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_data import mail_thunder_content_data_dict
# env
from je_mail_thunder.utils.save_mail_user_content.save_on_env import set_mail_thunder_os_environ
from je_mail_thunder.utils.save_mail_user_content.save_on_env import get_mail_thunder_os_environ
